"""
Random User-Agent
Copyright: 2022-2024 Ekin Karadeniz (github.com/iamdual)
License: Apache License 2.0
"""
from . import user_agent


def generate(**kwargs):
    return user_agent.UserAgent(**kwargs)



"""
Random User-Agent
Copyright: 2022 Ekin Karadeniz (github.com/iamdual)
License: Apache License 2.0 
"""
import unittest

import src.ua_generator as ua_generator
from src.ua_generator import exceptions


def raised_call():
    ua_generator.generate(device='desktop', platform='commodore_64')


def raised_call_2():
    ua_generator.generate(browser=('netscape', 'ie'))


class TestExceptions(unittest.TestCase):
    def test_value_error(self):
        self.assertRaises(exceptions.InvalidArgumentError, raised_call)

    def test_value_error_2(self):
        self.assertRaises(exceptions.InvalidArgumentError, raised_call_2)


if __name__ == '__main__':
    unittest.main()



"""
Random User-Agent
Copyright: 2022 Ekin Karadeniz (github.com/iamdual)
License: Apache License 2.0 
"""
import unittest

from src.ua_generator import utils


class TestUtils(unittest.TestCase):
    def test_contains(self):
        self.assertTrue(utils.contains(('mobile', 'desktop'), 'mobile'))
        self.assertTrue(utils.contains(('mobile', 'desktop'), 'desktop'))
        self.assertTrue(utils.contains('desktop', 'desktop'))
        self.assertFalse(utils.contains(('mobile', 'desktop'), 'ebook'))
        self.assertFalse(utils.contains('desktop', 'ebook'))

    def test_contains_multiple(self):
        self.assertTrue(utils.contains_multiple(('mobile', 'desktop'), ['mobile', 'desktop']))
        self.assertTrue(utils.contains_multiple(('mobile', 'desktop'), ['ebook', 'desktop']))
        self.assertTrue(utils.contains_multiple(['mobile', 'desktop'], ['ebook', 'mobile']))
        self.assertTrue(utils.contains_multiple('desktop', ('ebook', 'desktop')))
        self.assertTrue(utils.contains_multiple('desktop', ('ebook', 'desktop')))
        self.assertFalse(utils.contains_multiple(['mobile', 'desktop'], ['ebook', 'gameboy']))
        self.assertFalse(utils.contains_multiple('mobile', ['ebook', 'gameboy']))


if __name__ == '__main__':
    unittest.main()



"""
Random User-Agent
Copyright: 2022-2024 Ekin Karadeniz (github.com/iamdual)
License: Apache License 2.0
"""


def version(version_dict, strip_zero=False):
    v_str = str(version_dict['major'])

    if 'minor' in version_dict and (
            strip_zero is False or
            (strip_zero is True and version_dict['minor'] > 0)):
        v_str += '.' + str(version_dict['minor'])

    return v_str


def major_version(version_dict):
    return str(version_dict['major']).split('.', 1)[0]



"""
Random User-Agent
Copyright: 2022 Ekin Karadeniz (github.com/iamdual)
License: Apache License 2.0 
"""
import unittest

import src.ua_generator as ua_generator


class TestClientHints(unittest.TestCase):
    def test_ch_platform(self):
        for i in range(0, 100):
            ua = ua_generator.generate(browser=('chrome', 'edge'), platform='macos')
            self.assertIsNotNone(ua.ch)
            self.assertEqual(ua.ch.platform, '"macOS"')

    def test_ch_platform_2(self):
        for i in range(0, 100):
            ua = ua_generator.generate(browser=('chrome', 'edge'), platform='linux')
            self.assertIsNotNone(ua.ch)
            self.assertEqual(ua.ch.platform, '"Linux"')

    def test_ch_platform_version(self):
        for i in range(0, 100):
            ua = ua_generator.generate(browser=('chrome', 'edge'))
            self.assertIsNotNone(ua.ch)
            self.assertTrue(type(ua.ch.platform_version) is str)
            self.assertTrue(len(ua.ch.platform_version) > 0)

    def test_ch_platform_version_windows(self):
        for i in range(0, 100):
            ua = ua_generator.generate(platform='windows')
            self.assertIsNotNone(ua.ch)
            self.assertTrue(type(ua.ch.platform_version) is str)
            self.assertEqual(len(ua.ch.platform_version.split('.')), 3)

    def test_ch_mobile(self):
        for i in range(0, 100):
            ua = ua_generator.generate(browser=('chrome', 'edge'), platform='android')
            self.assertIsNotNone(ua.ch)
            self.assertEqual(ua.ch.mobile, '?1')

    def test_ch_non_mobile(self):
        for i in range(0, 100):
            ua = ua_generator.generate(browser=('chrome', 'edge'), platform='windows')
            self.assertIsNotNone(ua.ch)
            self.assertEqual(ua.ch.mobile, '?0')

    def test_ch_brands(self):
        for i in range(0, 100):
            ua = ua_generator.generate(browser='chrome', platform='windows')
            self.assertIsNotNone(ua.ch)
            self.assertTrue(ua.ch.brands.startswith('"Not A(Brand";v="99"'))
            self.assertTrue('Chromium' in ua.ch.brands)
            self.assertTrue('Google Chrome' in ua.ch.brands)

    def test_ch_brands_full_version_list(self):
        for i in range(0, 100):
            ua = ua_generator.generate(browser='edge', platform='windows')
            self.assertIsNotNone(ua.ch)
            self.assertTrue(ua.ch.brands_full_version_list.startswith('"Not A(Brand";v="99"'))
            self.assertTrue('Chromium' in ua.ch.brands_full_version_list)
            self.assertTrue('Microsoft Edge' in ua.ch.brands_full_version_list)

    def test_ch_bitness(self):
        for i in range(0, 100):
            ua = ua_generator.generate(browser=('chrome', 'edge'))
            self.assertIsNotNone(ua.ch)
            self.assertIn(ua.ch.bitness, ('"32"', '"64"'))

    def test_ch_architecture(self):
        for i in range(0, 100):
            ua = ua_generator.generate(browser=('chrome', 'edge'))
            self.assertIsNotNone(ua.ch)
            self.assertIn(ua.ch.architecture, ('"arm"', '"x86"'))


if __name__ == '__main__':
    unittest.main()



"""
Random User-Agent
Copyright: 2022-2024 Ekin Karadeniz (github.com/iamdual)
License: Apache License 2.0 
"""
import random

# User agent cap on macOS
# https://groups.google.com/a/chromium.org/g/blink-dev/c/hAI4QoX6rEo/m/qQNPThr0AAAJ

# https://support.apple.com/en-us/HT201222
versions = {
    '10.11': {'minor_range': (0, 6)},
    '10.12': {'minor_range': (0, 6)},
    '10.13': {'minor_range': (0, 6)},
    '10.14': {'minor_range': (0, 6)},
    '10.15': {'minor_range': (0, 7)},
    '11.0': {'minor_range': (0, 0)},
    '11.2': {'minor_range': (0, 3)},
    '11.3': {'minor_range': (0, 1)},
    '11.5': {'minor_range': (0, 2)},
    '11.6': {'minor_range': (0, 6)},
    '12.0': {'minor_range': (0, 1)},
    '12.2': {'minor_range': (0, 1)},
    '12.3': {'minor_range': (0, 1)},
    '12.4': {'minor_range': (0, 0)},
    '12.5': {'minor_range': (0, 1)},
    '12.6': {'minor_range': (0, 4)},
    '13.0': {'minor_range': (0, 1)},
    '13.1': {'minor_range': (0, 0)},
    '13.2': {'minor_range': (0, 1)},
    '13.3': {'minor_range': (0, 1)},
    '13.4': {'minor_range': (0, 1)},
    '13.5': {'minor_range': (0, 2)},
    '14.0': {'minor_range': (0, 1)},
    '14.1': {'minor_range': (0, 2)},
    '14.2': {'minor_range': (0, 1)},
    '14.3': {'minor_range': (0, 1)},
}


def get_version():
    choice = random.randint(0, len(versions) - 1)
    i = 0
    for major, props in versions.items():
        if choice == i:
            minor = random.randint(int(props['minor_range'][0]), int(props['minor_range'][1]))
            return {'major': major, 'minor': minor}
        i = i + 1



"""
Random User-Agent
Copyright: 2022-2024 Ekin Karadeniz (github.com/iamdual)
License: Apache License 2.0
"""
from . import formats, serialization, utils
from .data import platforms_mobile
from .data import generator


# https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Sec-CH-UA
# https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Sec-CH-UA-Full-Version-List
class ClientHints:
    # Type hinting only
    mobile: str
    platform: str
    platform_version: str
    brands: str
    brands_full_version_list: str
    bitness: str
    architecture: str

    def __init__(self, gen: generator.Generator):
        self.__generator = gen
        self.__cache = {}

    def __mobile(self):
        return self.__generator.platform in platforms_mobile

    def __platform(self):
        platform = self.__generator.platform

        if platform == 'ios':
            platform = 'iOS'
        elif platform == 'macos':
            platform = 'macOS'
        else:
            platform = platform.title()

        return platform

    def __platform_version(self):
        if self.__generator.platform == 'windows' and formats.major_version(self.__generator.platform_version) == '10':
            return utils.choice(('10.0.0', '13.0.0'))

        return formats.version(self.__generator.platform_version)

    def __brands(self, full_version_list: bool = False):
        brand_list = [{'brand': 'Not A(Brand', 'version': '99'}]

        if full_version_list:
            browser_version = formats.version(self.__generator.browser_version)
        else:
            browser_version = formats.major_version(self.__generator.browser_version)

        if self.__generator.browser == 'chrome':
            brand_list.append({'brand': 'Chromium', 'version': browser_version})
            brand_list.append({'brand': 'Google Chrome', 'version': browser_version})
        elif self.__generator.browser == 'edge':
            brand_list.append({'brand': 'Chromium', 'version': browser_version})
            brand_list.append({'brand': 'Microsoft Edge', 'version': browser_version})

        return brand_list

    def __bitness(self):
        if self.__generator.platform == 'android':
            return utils.choice(('32', '64', '32', '32'))

        return '64'

    def __architecture(self):
        if self.__generator.platform == 'android' or self.__generator.platform == 'ios':
            return 'arm'
        elif self.__generator.platform == 'macos':
            return utils.choice(('arm', 'x86', 'arm', 'arm'))

        return 'x86'

    def __getattr__(self, name):
        if name in self.__cache:
            return self.__cache[name]

        if name == 'mobile':
            self.__cache[name] = serialization.ch_bool(self.__mobile())
        elif name == 'platform':
            self.__cache[name] = serialization.ch_string(self.__platform())
        elif name == 'platform_version':
            self.__cache[name] = serialization.ch_string(self.__platform_version())
        elif name == 'brands':
            self.__cache[name] = serialization.ch_brand_list(self.__brands())
        elif name == 'brands_full_version_list':
            self.__cache[name] = serialization.ch_brand_list(self.__brands(full_version_list=True))
        elif name == 'bitness':
            self.__cache[name] = serialization.ch_string(self.__bitness())
        elif name == 'architecture':
            self.__cache[name] = serialization.ch_string(self.__architecture())

        return self.__cache[name]

    def __str__(self):
        return self.brands



"""
Random User-Agent
Copyright: 2022-2024 Ekin Karadeniz (github.com/iamdual)
License: Apache License 2.0 
"""
import random

# https://developer.apple.com/documentation/ios-ipados-release-notes
# https://support.apple.com/en-us/HT201222
versions = {
    '14.0': {'minor_range': (0, 1)},
    '14.1': {'minor_range': (0, 0)},
    '14.2': {'minor_range': (0, 1)},
    '14.3': {'minor_range': (0, 0)},
    '14.4': {'minor_range': (0, 2)},
    '14.5': {'minor_range': (0, 1)},
    '14.6': {'minor_range': (0, 0)},
    '14.7': {'minor_range': (0, 1)},
    '15.0': {'minor_range': (0, 2)},
    '15.1': {'minor_range': (0, 1)},
    '15.2': {'minor_range': (0, 1)},
    '15.3': {'minor_range': (0, 1)},
    '15.4': {'minor_range': (0, 1)},
    '15.5': {'minor_range': (0, 0)},
    '15.6': {'minor_range': (0, 1)},
    '16.4': {'minor_range': (0, 1)},
    '16.5': {'minor_range': (0, 2)},
    '16.6': {'minor_range': (0, 1)},
    '17.0': {'minor_range': (0, 3)},
    '17.1': {'minor_range': (0, 2)},
    '17.2': {'minor_range': (0, 1)},
    '17.3': {'minor_range': (0, 1)},
}


def get_version():
    choice = random.randint(0, len(versions) - 1)
    i = 0
    for major, props in versions.items():
        if choice == i:
            minor = random.randint(int(props['minor_range'][0]), int(props['minor_range'][1]))
            return {'major': major, 'minor': minor}
        i = i + 1
